'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var childProcess = require('child_process');
var logger = require('./logger');

var STDIO_DEFAULT = 'inherit';
var STDIO_PIPE = 'pipe';

// 封装spawn方法
function spawnExec(command, options) {
  var commands = command.split(' ');
  var cmd = commands[0];
  var args = commands.slice(1);

  if (logger.debugMode) {
    args.push('--debug');
  }

  if (process.platform === 'win32') {
    cmd += '.cmd';
  }

  logger.debug('worker\u547D\u4EE4\u89E3\u6790\u7ED3\u679C: (' + cmd + ', [' + args + '])');
  return childProcess.spawn(cmd, args, options);
}

/**
 * 任务流
 * @param  {[Array]} tasks [任务列表]
 *     @param {[String]} command [命令]
 *     @param {[Object]} options [选项]
 *        @param {[String]} stdio [子进程输出类型]
 *        @param {[String]} cwd [执行环境]
 * @param  {[Boolean]} sync  [是否同步执行]
 */
module.exports = function (tasks, sync) {
  if (!Array.isArray(tasks)) {
    var message = '执行任务流传入非法参数';
    logger.error(message);
    var err = new Error();
    err.message = message;
    err.code = 'INVALID';
    throw err;
  }
  if (sync) {
    exports.commands(tasks);
  } else {
    exports.workers(tasks);
  }
};

// 命令
exports.commands = function (tasks) {
  logger.debug('开始执行命令');
  tasks.forEach(function (task) {
    logger.debug('command : ' + task.command);

    var opts = task.options || {};
    opts.stdio = opts.stdio || STDIO_DEFAULT;

    logger.debug('\u547D\u4EE4\u7684\u8F93\u5165\u8F93\u51FA\u6807\u51C6 : ' + opts.stdio);

    childProcess.execSync(task.command, opts);
  });
};

// 任务
exports.workers = function (tasks) {
  if (!tasks.length) {
    return false;
  }

  var task = tasks.splice(0, 1)[0];
  var opts = task.options || {};
  opts.env = (0, _assign2.default)({}, process.env, opts.env);
  opts.stdio = opts.stdio || STDIO_DEFAULT;
  opts.cwd = opts.cwd || process.cwd();

  logger.debug('worker : ' + task.command);
  logger.debug('worker\u7684\u8F93\u5165\u8F93\u51FA\u6807\u51C6 : ' + opts.stdio);

  if (task.monitor && typeof task.monitor === 'string') {
    opts.stdio = STDIO_PIPE;
    logger.debug('\u914D\u7F6Emonitor\u573A\u666F\u7684\u8F93\u5165\u8F93\u51FA\u6807\u51C6\u53D8\u66F4\u4E3A : ' + opts.stdio);
    var ps = spawnExec(task.command, opts);
    ps.stdout.on('data', function (trunk) {
      var line = trunk.toString();
      logger.plain(line);
      if (~line.indexOf(task.monitor)) {
        exports.workers(tasks);
      }
    });
  } else {
    spawnExec(task.command, opts);
    exports.workers(tasks);
  }
  return true;
};